@@ -11,7 +11,7 @@
 # under the License.
 
 from openstack.cluster import cluster_service
-from openstack import resource
+from openstack import resource2 as resource
 from openstack import utils
 
 
@@ -23,53 +23,56 @@ class Node(resource.Resource):
 
     # capabilities
     allow_create = True
-    allow_retrieve = True
+    allow_get = True
     allow_update = True
     allow_delete = True
     allow_list = True
 
     patch_update = True
 
+    _query_mapping = resource.QueryParameters(
+        'show_details', 'name', 'sort', 'global_project', 'cluster_id')
+
     # Properties
     #: The name of the node.
-    name = resource.prop('name')
+    name = resource.Body('name')
     #: The ID of the physical object that backs the node.
-    physical_id = resource.prop('physical_id')
+    physical_id = resource.Body('physical_id')
     #: The ID of the cluster in which this node is a member.
     #: A node is an orphan node if this field is empty.
-    cluster_id = resource.prop('cluster_id')
+    cluster_id = resource.Body('cluster_id')
     #: The ID of the profile used by this node.
-    profile_id = resource.prop('profile_id')
+    profile_id = resource.Body('profile_id')
     #: The ID of the project this node belongs to.
-    project_id = resource.prop('project')
+    project_id = resource.Body('project')
     #: The name of the profile used by this node.
-    profile_name = resource.prop('profile_name')
+    profile_name = resource.Body('profile_name')
     #: An integer that is unique inside the owning cluster.
     #: A value of -1 means this node is an orphan node.
-    index = resource.prop('index', type=int)
+    index = resource.Body('index', type=int)
     #: A string indicating the role the node plays in a cluster.
-    role = resource.prop('role')
+    role = resource.Body('role')
     #: The timestamp of the node object's initialization.
     #: *Type: datetime object parsed from ISO 8601 formatted string*
-    init_at = resource.prop('init_at')
+    init_at = resource.Body('init_at')
     #: The timestamp of the node's creation, i.e. the physical object
     #: represented by this node is also created.
     #: *Type: datetime object parsed from ISO 8601 formatted string*
-    created_at = resource.prop('created_at')
+    created_at = resource.Body('created_at')
     #: The timestamp the node was last updated.
     #: *Type: datetime object parsed from ISO 8601 formatted string*
-    updated_at = resource.prop('updated_at')
+    updated_at = resource.Body('updated_at')
     #: A string indicating the node's status.
-    status = resource.prop('status')
+    status = resource.Body('status')
     #: A string describing why the node entered its current status.
-    status_reason = resource.prop('status_reason')
+    status_reason = resource.Body('status_reason')
     #: A map containing key-value pairs attached to the node.
-    metadata = resource.prop('tags', type=dict)
+    metadata = resource.Body('tags', type=dict)
     #: A map containing some runtime data for this node.
-    data = resource.prop('data', type=dict)
+    data = resource.Body('data', type=dict)
     #: A map containing the details of the physical object this node
     #: represents
-    details = resource.prop('details', type=dict)
+    details = resource.Body('details', type=dict)
 
     def _action(self, session, body):
         """Procedure the invoke an action API.
@@ -103,16 +106,14 @@ class Node(resource.Resource):
         }
         return self._action(session, body)
 
-    def delete(self, session):
-        """Delete the remote resource associated with this instance.
 
-        :param session: The session to use for making this request.
-        :type session: :class:`~openstack.session.Session`
+class NodeDetail(Node):
+    base_path = '/nodes/%(node_id)s?show_details=True'
 
-        :returns: The instance of the Node which was deleted.
-        :rtype: :class:`~openstack.cluster.v1.node.Node`.
-        """
-        url = self._get_url(self, self.id)
-        resp = session.delete(url, endpoint_filter=self.service)
-        self.location = resp.headers['location']
-        return self
+    allow_create = False
+    allow_get = True
+    allow_update = False
+    allow_delete = False
+    allow_list = False
+
+    node_id = resource.URI('node_id')
