@@ -21,11 +21,11 @@ from openstack.compute.v2 import server as _server
 from openstack.compute.v2 import server_group as _server_group
 from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
-from openstack import proxy
-from openstack import resource
+from openstack import proxy2
+from openstack import resource2
 
 
-class Proxy(proxy.BaseProxy):
+class Proxy(proxy2.BaseProxy):
 
     def find_extension(self, name_or_id, ignore_missing=True):
         """Find a single extension
@@ -42,16 +42,13 @@ class Proxy(proxy.BaseProxy):
         return self._find(extension.Extension, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def extensions(self, **query):
+    def extensions(self):
         """Retrieve a generator of extensions
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-                                 the resources being returned.
-
         :returns: A generator of extension instances.
         :rtype: :class:`~openstack.compute.v2.extension.Extension`
         """
-        return self._list(extension.Extension, paginated=False, **query)
+        return self._list(extension.Extension, paginated=False)
 
     def find_flavor(self, name_or_id, ignore_missing=True):
         """Find a single flavor
@@ -121,19 +118,6 @@ class Proxy(proxy.BaseProxy):
         flv = _flavor.FlavorDetail if details else _flavor.Flavor
         return self._list(flv, paginated=True, **query)
 
-    def update_flavor(self, flavor, **attrs):
-        """Update a flavor
-
-        :param flavor: Either the ID of a flavor or a
-                      :class:`~openstack.compute.v2.flavor.Flavor` instance.
-        :attrs kwargs: The attributes to update on the flavor represented
-                       by ``value``.
-
-        :returns: The updated flavor
-        :rtype: :class:`~openstack.compute.v2.flavor.Flavor`
-        """
-        return self._update(_flavor.Flavor, flavor, **attrs)
-
     def delete_image(self, image, ignore_missing=True):
         """Delete an image
 
@@ -197,7 +181,7 @@ class Proxy(proxy.BaseProxy):
         if isinstance(res, base):
             return res
         else:
-            return base({"id": res})
+            return base(id=res)
 
     def get_image_metadata(self, image):
         """Return a dictionary of metadata for an image
@@ -308,30 +292,13 @@ class Proxy(proxy.BaseProxy):
         return self._find(_keypair.Keypair, name_or_id,
                           ignore_missing=ignore_missing)
 
-    def keypairs(self, **query):
+    def keypairs(self):
         """Return a generator of keypairs
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-                                 the resources being returned.
-
         :returns: A generator of keypair objects
         :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
         """
-        return self._list(_keypair.Keypair, paginated=False, **query)
-
-    def update_keypair(self, keypair, **attrs):
-        """Update a keypair
-
-        :param keypair: Either the ID of a keypair or a
-                        :class:`~openstack.compute.v2.keypair.Keypair`
-                        instance.
-        :attrs kwargs: The attributes to update on the keypair represented
-                       by ``keypair``.
-
-        :returns: The updated keypair
-        :rtype: :class:`~openstack.compute.v2.keypair.Keypair`
-        """
-        return self._update(_keypair.Keypair, keypair, **attrs)
+        return self._list(_keypair.Keypair, paginated=False)
 
     def get_limits(self):
         """Retrieve limits that are applied to the project's account
@@ -434,12 +401,6 @@ class Proxy(proxy.BaseProxy):
         :returns: A generator of server instances.
         """
         srv = _server.ServerDetail if details else _server.Server
-
-        # Server expects changes-since, but we use an underscore
-        # so it can be a proper Python name.
-        if "changes_since" in query:
-            query["changes-since"] = query.pop("changes_since")
-
         return self._list(srv, paginated=True, **query)
 
     def update_server(self, server, **attrs):
@@ -455,10 +416,142 @@ class Proxy(proxy.BaseProxy):
         """
         return self._update(_server.Server, server, **attrs)
 
+    def change_server_password(self, server, new_password):
+        """Change the administrator password
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param str new_password: The new password to be set.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        server.change_password(self.session, new_password)
+
+    def reboot_server(self, server, reboot_type):
+        """Reboot a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param str reboot_type: The type of reboot to perform.
+                                "HARD" and "SOFT" are the current options.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        server.reboot(self.session, reboot_type)
+
+    def rebuild_server(self, server, name, admin_password, **attrs):
+        """Rebuild a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param str name: The name of the server
+        :param str admin_password: The administrator password
+        :param bool preserve_ephemeral: Indicates whether the server
+            is rebuilt with the preservation of the ephemeral partition.
+            *Default: False*
+        :param str image: The id of an image to rebuild with. *Default: None*
+        :param str access_ipv4: The IPv4 address to rebuild with.
+                                *Default: None*
+        :param str access_ipv6: The IPv6 address to rebuild with.
+                                *Default: None*
+        :param dict metadata: A dictionary of metadata to rebuild with.
+                               *Default: None*
+        :param list personality: A list of dictionaries, each including a
+                                 **path** and **contents** key, to be injected
+                                 into the rebuilt server at launch.
+                                 *Default: None*
+
+        :returns: The rebuilt :class:`~openstack.compute.v2.server.Server`
+                  instance.
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.rebuild(self.session, name, admin_password, **attrs)
+
+    def resize_server(self, server, flavor):
+        """Resize a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param flavor: Either the ID of a flavor or a
+                       :class:`~openstack.compute.v2.flavor.Flavor` instance.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        flavor_id = resource2.Resource._get_id(flavor)
+        server.resize(self.session, flavor_id)
+
+    def confirm_server_resize(self, server):
+        """Confirm a server resize
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        server.confirm_resize(self.session)
+
+    def revert_server_resize(self, server):
+        """Revert a server resize
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        server.revert_resize(self.session)
+
+    def create_server_image(self, server, name, metadata=None):
+        """Create an image from a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param str name: The name of the image to be created.
+        :param dict metadata: A dictionary of metadata to be set on the image.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        server.create_image(self.session, name, metadata)
+
+    def add_security_group_to_server(self, server, security_group):
+        """Add a security group to a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param security_group: Either the ID of a security group or a
+            :class:`~openstack.network.v2.security_group.SecurityGroup`
+            instance.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        security_group_id = resource2.Resource._get_id(security_group)
+        server.add_security_group(self.session, security_group_id)
+
+    def remove_security_group_from_server(self, server, security_group):
+        """Add a security group to a server
+
+        :param server: Either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server` instance.
+        :param security_group: Either the ID of a security group or a
+            :class:`~openstack.network.v2.security_group.SecurityGroup`
+            instance.
+
+        :returns: None
+        """
+        server = self._get_resource(_server.Server, server)
+        security_group_id = resource2.Resource._get_id(security_group)
+        server.remove_security_group(self.session, security_group_id)
+
     def wait_for_server(self, server, status='ACTIVE', failures=['ERROR'],
                         interval=2, wait=120):
-        return resource.wait_for_status(self.session, server, status,
-                                        failures, interval, wait)
+        return resource2.wait_for_status(self.session, server, status,
+                                         failures, interval, wait)
 
     def create_server_interface(self, server, **attrs):
         """Create a new server interface from attributes
@@ -473,9 +566,9 @@ class Proxy(proxy.BaseProxy):
         :returns: The results of server interface creation
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource.Resource.get_id(server)
+        server_id = resource2.Resource._get_id(server)
         return self._create(_server_interface.ServerInterface,
-                            path_args={'server_id': server_id}, **attrs)
+                            server_id=server_id, **attrs)
 
     def delete_server_interface(self, server_interface, server=None,
                                 ignore_missing=True):
@@ -497,13 +590,13 @@ class Proxy(proxy.BaseProxy):
 
         :returns: ``None``
         """
-        if isinstance(server_interface, _server_interface.ServerInterface):
-            server_id = server_interface.server_id
-        else:
-            server_id = resource.Resource.get_id(server)
+        server_id = self._get_uri_attribute(server_interface, server,
+                                            "server_id")
+        server_interface = resource2.Resource._get_id(server_interface)
 
-        self._delete(_server_interface.ServerInterface, server_interface,
-                     path_args={'server_id': server_id},
+        self._delete(_server_interface.ServerInterface,
+                     port_id=server_interface,
+                     server_id=server_id,
                      ignore_missing=ignore_missing)
 
     def get_server_interface(self, server_interface, server=None):
@@ -523,131 +616,58 @@ class Proxy(proxy.BaseProxy):
         :raises: :class:`~openstack.exceptions.ResourceNotFound`
                  when no resource can be found.
         """
-        if isinstance(server_interface, _server_interface.ServerInterface):
-            server_id = server_interface.server_id
-        else:
-            server_id = resource.Resource.get_id(server)
+        server_id = self._get_uri_attribute(server_interface, server,
+                                            "server_id")
+        server_interface = resource2.Resource._get_id(server_interface)
 
-        return self._get(_server_interface.ServerInterface, server_interface,
-                         path_args={'server_id': server_id})
+        return self._get(_server_interface.ServerInterface,
+                         server_id=server_id, port_id=server_interface)
 
-    def server_interfaces(self, server, **query):
+    def server_interfaces(self, server):
         """Return a generator of server interfaces
 
         :param server: The server can be either the ID of a server or a
                        :class:`~openstack.compute.v2.server.Server`.
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-                                 the resources being returned.
 
         :returns: A generator of ServerInterface objects
         :rtype: :class:`~openstack.compute.v2.server_interface.ServerInterface`
         """
-        server_id = resource.Resource.get_id(server)
+        server_id = resource2.Resource._get_id(server)
         return self._list(_server_interface.ServerInterface, paginated=False,
-                          path_args={'server_id': server_id},
-                          **query)
+                          server_id=server_id)
 
-    def find_server_ip(self, name_or_id, ignore_missing=True):
-        """Find a single server IP
-
-        :param name_or_id: The name or ID of a server IP.
-        :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
-                    raised when the resource does not exist.
-                    When set to ``True``, None will be returned when
-                    attempting to find a nonexistent resource.
-        :returns: One :class:`~openstack.compute.v2.server_ip.ServerIP` or None
-        """
-        return self._find(server_ip.ServerIP, name_or_id,
-                          ignore_missing=ignore_missing)
-
-    def server_ips(self, **query):
+    def server_ips(self, server, network_label=None):
         """Return a generator of server IPs
 
-        :param kwargs \*\*query: Optional query parameters to be sent to limit
-                                 the resources being returned.
+        :param server: The server can be either the ID of a server or a
+                       :class:`~openstack.compute.v2.server.Server`.
+        :param network_label: The name of a particular network to list
+                              IP addresses from.
 
         :returns: A generator of ServerIP objects
         :rtype: :class:`~openstack.compute.v2.server_ip.ServerIP`
         """
-        return self._list(server_ip.ServerIP, paginated=False, **query)
+        server_id = resource2.Resource._get_id(server)
+        return self._list(server_ip.ServerIP, paginated=False,
+                          server_id=server_id, network_label=network_label)
 
-    def resize_server(self, server, flavor):
-        """Resize a server
-
-        :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param falvor: The ID or name of the flavor used to resize the server.
-
-        :returns: None
-        """
-        server = _server.Server.from_id(server)
-        server.resize(self.session, flavor)
-
-    def confirm_resize_server(self, server):
-        """Confirm a pending resize_server action
-
-        :param server: Either the ID of a server or a
-                      :class:`~openstack.compute.v2.server.Server` instance.
-
-        :returns: None
-        """
-        server = _server.Server.from_id(server)
-        server.confirm_resize(self.session)
-
-    def revert_resize_server(self, server):
-        """Cancel and revert a pending resize_server action
-
-        :param server: Either the ID of a server or a
-                      :class:`~openstack.compute.v2.server.Server` instance.
-
-        :returns: None
-        """
-        server = _server.Server.from_id(server)
-        server.revert_resize(self.session)
-
-    def rebuild_server(self, server, image, name=None, admin_password=None,
-                       **attrs):
-        """Rebuild a server
-
-        :param server: Either the ID of a server or a
-                      :class:`~openstack.compute.v2.server.Server` instance.
-        :param image: The ID or name or a
-                      :class:`~openstack.compute.v2.image.Image` or full
-                      URL of the image used to rebuild the server with.
-        :param name: New name for the server.
-        :param admin_password: New admin password for the server.
-        :param kwargs \*\*attrs: The attributes to rebuild the server.
-
-        :returns: The rebuilt server
-        :rtype: :class:`~openstack.compute.v2.server.Server`
-        """
-        if isinstance(image, _image.Image):
-            image_ref = image.id
-        else:
-            image_obj = self.find_image(image)
-            if image_obj:
-                image_ref = image_obj.id
-            else:
-                # the 'image' could be a full url
-                image_ref = image
-
-        server = _server.Server.from_id(server)
-        return server.rebuild(self.session, name, image_ref, admin_password,
-                              **attrs)
-
-    def availability_zones(self, **query):
+    def availability_zones(self, details=False):
         """Return a generator of availability zones
 
-        :param kwargs \*\*query: Optional query parameters to be sent
-                                 to limit the resources being returned.
+        :param bool details: Return extra details about the availability
+                             zones. This defaults to `False` as it generally
+                             requires extra permission.
 
         :returns: A generator of availability zone
         :rtype: :class:`~openstack.compute.v2.availability_zone.
         AvailabilityZone`
         """
-        return self._list(availability_zone.AvailabilityZone,
-                          paginated=False, **query)
+        if details:
+            az = availability_zone.AvailabilityZoneDetail
+        else:
+            az = availability_zone.AvailabilityZone
+
+        return self._list(az, paginated=False)
 
     def get_server_metadata(self, server):
         """Return a dictionary of metadata for a server
@@ -773,14 +793,14 @@ class Proxy(proxy.BaseProxy):
         """
         return self._list(_server_group.ServerGroup, paginated=False, **query)
 
-    def hypervisors(self, **query):
+    def hypervisors(self):
         """Return a generator of hypervisor
 
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
 
-        return self._list(_hypervisor.Hypervisor, paginated=False, **query)
+        return self._list(_hypervisor.Hypervisor, paginated=False)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
