@@ -409,6 +409,7 @@ class TestResource(base.TestCase):
         self.assertFalse(sot.allow_list)
         self.assertFalse(sot.allow_head)
         self.assertFalse(sot.patch_update)
+        self.assertFalse(sot.put_create)
 
     def test_repr(self):
         a = {"a": 1}
@@ -438,6 +439,19 @@ class TestResource(base.TestCase):
         self.assertIn("b=2", the_repr)
         self.assertIn("c=3", the_repr)
 
+    def test_equality(self):
+        class Example(resource2.Resource):
+            x = resource2.Body("x")
+            y = resource2.Header("y")
+            z = resource2.URI("z")
+
+        e1 = Example(x=1, y=2, z=3)
+        e2 = Example(x=1, y=2, z=3)
+        e3 = Example(x=0, y=0, z=0)
+
+        self.assertEqual(e1, e2)
+        self.assertNotEqual(e1, e3)
+
     def test__update(self):
         sot = resource2.Resource()
 
@@ -559,9 +573,19 @@ class TestResource(base.TestCase):
 
     def test__alternate_id(self):
         class Test(resource2.Resource):
-            alt = resource2.Body("alt", alternate_id=True)
+            alt = resource2.Body("the_alt", alternate_id=True)
 
-        self.assertTrue("alt", Test._alternate_id())
+        self.assertTrue("the_alt", Test._alternate_id())
+
+        value1 = "lol"
+        sot = Test(alt=value1)
+        self.assertEqual(sot.alt, value1)
+        self.assertEqual(sot.id, value1)
+
+        value2 = "rofl"
+        sot = Test(the_alt=value2)
+        self.assertEqual(sot.alt, value2)
+        self.assertEqual(sot.id, value2)
 
     def test__get_id_instance(self):
         class Test(resource2.Resource):
@@ -585,6 +609,125 @@ class TestResource(base.TestCase):
         value = "id"
         self.assertEqual(value, resource2.Resource._get_id(value))
 
+    def test_to_dict(self):
+
+        class Test(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        res = Test(id='FAKE_ID')
+
+        expected = {
+            'id': 'FAKE_ID',
+            'name': None,
+            'location': None,
+            'foo': None,
+            'bar': None
+        }
+        self.assertEqual(expected, res.to_dict())
+
+    def test_to_dict_no_body(self):
+
+        class Test(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        res = Test(id='FAKE_ID')
+
+        expected = {
+            'location': None,
+            'foo': None,
+        }
+        self.assertEqual(expected, res.to_dict(body=False))
+
+    def test_to_dict_no_header(self):
+
+        class Test(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        res = Test(id='FAKE_ID')
+
+        expected = {
+            'id': 'FAKE_ID',
+            'name': None,
+            'bar': None
+        }
+        self.assertEqual(expected, res.to_dict(headers=False))
+
+    def test_to_dict_ignore_none(self):
+
+        class Test(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        res = Test(id='FAKE_ID', bar='BAR')
+
+        expected = {
+            'id': 'FAKE_ID',
+            'bar': 'BAR',
+        }
+        self.assertEqual(expected, res.to_dict(ignore_none=True))
+
+    def test_to_dict_with_mro(self):
+
+        class Parent(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        class Child(Parent):
+            foo_new = resource2.Header('foo_baz_server')
+            bar_new = resource2.Body('bar_baz_server')
+
+        res = Child(id='FAKE_ID')
+
+        expected = {
+            'foo': None,
+            'bar': None,
+            'foo_new': None,
+            'bar_new': None,
+            'id': 'FAKE_ID',
+            'location': None,
+            'name': None
+        }
+        self.assertEqual(expected, res.to_dict())
+
+    def test_to_dict_value_error(self):
+
+        class Test(resource2.Resource):
+            foo = resource2.Header('foo')
+            bar = resource2.Body('bar')
+
+        res = Test(id='FAKE_ID')
+
+        err = self.assertRaises(ValueError,
+                                res.to_dict, body=False, headers=False)
+        self.assertEqual('At least one of `body` or `headers` must be True',
+                         six.text_type(err))
+
+    def test_to_dict_with_mro_no_override(self):
+
+        class Parent(resource2.Resource):
+            header = resource2.Header('HEADER')
+            body = resource2.Body('BODY')
+
+        class Child(Parent):
+            # The following two properties are not supposed to be overridden
+            # by the parent class property values.
+            header = resource2.Header('ANOTHER_HEADER')
+            body = resource2.Body('ANOTHER_BODY')
+
+        res = Child(id='FAKE_ID', body='BODY_VALUE', header='HEADER_VALUE')
+
+        expected = {
+            'body': 'BODY_VALUE',
+            'header': 'HEADER_VALUE',
+            'id': 'FAKE_ID',
+            'location': None,
+            'name': None
+        }
+        self.assertEqual(expected, res.to_dict())
+
     def test_new(self):
         class Test(resource2.Resource):
             attr = resource2.Body("attr")
@@ -649,19 +792,20 @@ class TestResource(base.TestCase):
         self.assertEqual({key: {"x": body_value}}, result.body)
         self.assertEqual({"y": header_value}, result.headers)
 
-    def test__transpose_component(self):
+    def test__filter_component(self):
         client_name = "client_name"
         server_name = "serverName"
         value = "value"
         # Include something in the mapping that we don't receive
         # so the branch that looks at existence in the compoment is checked.
         mapping = {client_name: server_name, "other": "blah"}
-        component = {server_name: value}
+        component = {server_name: value, "something": "else"}
 
         sot = resource2.Resource()
-        result = sot._transpose_component(component, mapping)
+        result = sot._filter_component(component, mapping)
 
-        self.assertEqual({client_name: value}, result)
+        # The something:else mapping should not make it into here.
+        self.assertEqual({server_name: value}, result)
 
     def test__translate_response_no_body(self):
         class Test(resource2.Resource):
@@ -671,7 +815,7 @@ class TestResource(base.TestCase):
         response.headers = dict()
 
         sot = Test()
-        sot._transpose_component = mock.Mock(return_value={"attr": "value"})
+        sot._filter_component = mock.Mock(return_value={"attr": "value"})
 
         sot._translate_response(response, has_body=False)
 
@@ -688,7 +832,7 @@ class TestResource(base.TestCase):
         response.json.return_value = body
 
         sot = Test()
-        sot._transpose_component = mock.Mock(side_effect=[body, dict()])
+        sot._filter_component = mock.Mock(side_effect=[body, dict()])
 
         sot._translate_response(response, has_body=True)
 
@@ -709,7 +853,7 @@ class TestResource(base.TestCase):
         response.json.return_value = {key: body}
 
         sot = Test()
-        sot._transpose_component = mock.Mock(side_effect=[body, dict()])
+        sot._filter_component = mock.Mock(side_effect=[body, dict()])
 
         sot._translate_response(response, has_body=True)
 
@@ -787,13 +931,15 @@ class TestResourceActions(base.TestCase):
         self.session.delete = mock.Mock(return_value=self.response)
         self.session.head = mock.Mock(return_value=self.response)
 
-    def _test_create(self, requires_id=False, prepend_key=False):
-        if not requires_id:
-            self.sot.id = None
+    def _test_create(self, cls, requires_id=False, prepend_key=False):
+        id = "id" if requires_id else None
+        sot = cls(id=id)
+        sot._prepare_request = mock.Mock(return_value=self.request)
+        sot._translate_response = mock.Mock()
 
-        result = self.sot.create(self.session)
+        result = sot.create(self.session)
 
-        self.sot._prepare_request.assert_called_once_with(
+        sot._prepare_request.assert_called_once_with(
             requires_id=requires_id, prepend_key=prepend_key)
         if requires_id:
             self.session.put.assert_called_once_with(
@@ -806,22 +952,43 @@ class TestResourceActions(base.TestCase):
                 endpoint_filter=self.service_name,
                 json=self.request.body, headers=self.request.headers)
 
-        self.sot._translate_response.assert_called_once_with(self.response)
-        self.assertEqual(result, self.sot)
+        sot._translate_response.assert_called_once_with(self.response)
+        self.assertEqual(result, sot)
 
-    def test_create_with_id(self):
-        self._test_create(requires_id=True, prepend_key=True)
+    def test_put_create(self):
+        class Test(resource2.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_create = True
+            put_create = True
 
-    def test_create_without_id(self):
-        self._test_create(requires_id=False, prepend_key=True)
+        self._test_create(Test, requires_id=True, prepend_key=True)
+
+    def test_post_create(self):
+        class Test(resource2.Resource):
+            service = self.service_name
+            base_path = self.base_path
+            allow_create = True
+            put_create = False
+
+        self._test_create(Test, requires_id=False, prepend_key=True)
 
     def test_get(self):
         result = self.sot.get(self.session)
 
-        self.sot._prepare_request.assert_called_once_with()
+        self.sot._prepare_request.assert_called_once_with(requires_id=True)
         self.session.get.assert_called_once_with(
-            self.request.uri,
-            endpoint_filter=self.service_name)
+            self.request.uri, endpoint_filter=self.service_name)
+
+        self.sot._translate_response.assert_called_once_with(self.response)
+        self.assertEqual(result, self.sot)
+
+    def test_get_not_requires_id(self):
+        result = self.sot.get(self.session, False)
+
+        self.sot._prepare_request.assert_called_once_with(requires_id=False)
+        self.session.get.assert_called_once_with(
+            self.request.uri, endpoint_filter=self.service_name)
 
         self.sot._translate_response.assert_called_once_with(self.response)
         self.assertEqual(result, self.sot)
@@ -845,7 +1012,7 @@ class TestResourceActions(base.TestCase):
         self.sot._body = mock.Mock()
         self.sot._body.dirty = mock.Mock(return_value={"x": "y"})
 
-        result = self.sot.update(self.session)
+        self.sot.update(self.session)
 
         self.sot._prepare_request.assert_called_once_with(prepend_key=True)
 
@@ -861,7 +1028,6 @@ class TestResourceActions(base.TestCase):
                 json=self.request.body, headers=self.request.headers)
 
         self.sot._translate_response.assert_called_once_with(self.response)
-        self.assertEqual(result, self.sot)
 
     def test_update_put(self):
         self._test_update(patch_update=False)
@@ -875,8 +1041,7 @@ class TestResourceActions(base.TestCase):
         self.sot._header = mock.Mock()
         self.sot._header.dirty = dict()
 
-        result = self.sot.update(self.session)
-        self.assertEqual(result, self.sot)
+        self.sot.update(self.session)
 
         self.session.put.assert_not_called()
 
